<?php

namespace app\admin\controller\order;

use app\admin\service\order\OrderService;
use app\admin\validate\coupon\CouponValidate;
use app\admin\validate\device\DeviceValidate;
use app\admin\validate\order\OrderExpressValidate;
use app\admin\validate\order\OrderValidate;
use app\controller\Curd;
use app\model\Appointment;
use app\model\Coupon;
use app\model\Device;
use app\model\Goods;
use app\model\GoodsComponent;
use app\model\Member;
use app\model\MemberAccount;
use app\model\MemberBenefit;
use app\model\Order;
use app\model\OrderExpress;
use app\model\OrderSheet;
use app\model\PayDetail;
use app\model\Premises;
use app\model\Supplier;
use app\model\SysDept;
use app\model\SysSerial;
use support\Db;
use support\exception\BusinessException;
use support\Redis;
use support\Request;
use support\Response;

class WholeController extends Curd
{
    public function __construct()
    {
        $this->model = new Order();
        $this->validate = true;
        $this->validateClass = new OrderValidate();
    }

    /**
     * @Desc 列表
     * @Author Gorden
     * @Date 2024/3/28 15:01
     *
     * @param Request $request
     * @return Response
     * @throws \support\exception\BusinessException
     */
    public function select(Request $request): Response
    {
        [$where, $format, $limit, $field, $order] = $this->selectInput($request);
        if (!empty($where['order_addtimes'])) {
            $where['order_addtimes'][0] = strtotime($where['order_addtimes'][0]);
            $where['order_addtimes'][1] = strtotime($where['order_addtimes'][1]);
        }
        $order = $request->get('order', 'desc');
        $field = $field ?? 'order_addtimes';
        if (!empty($where['order_status_system']) && $where['order_status_system'] == 'RETURN') {
            unset($where['order_status_system']);
            $where['order_category'] = 'RETURN';
        }
        $query = $this->doSelect($where, $field, $order);
        return $this->doFormat($query, $format, $limit);
    }

    protected function doSelect(array $where, string $field = null, string $order = 'desc')
    {
        $model = $this->model->with([
            'sheet' => function ($query) {
                $query->select('join_sheet_order_id', 'join_sheet_goods_id');
            },
            'member' => function ($query) {
                $query->select('member_id', 'member_mobile');
            }
        ]);
        foreach ($where as $column => $value) {
            if (is_array($value)) {
                if ($value[0] === 'like' || $value[0] === 'not like') {
                    $model = $model->where($column, $value[0], "%$value[1]%");
                } elseif (in_array($value[0], ['>', '=', '<', '<>'])) {
                    $model = $model->where($column, $value[0], $value[1]);
                } elseif ($value[0] == 'in' && !empty($value[1])) {
                    $valArr = $value[1];
                    if (is_string($value[1])) {
                        $valArr = explode(",", trim($value[1]));
                    }
                    $model = $model->whereIn($column, $valArr);
                } elseif ($value[0] == 'not in' && !empty($value[1])) {
                    $valArr = $value[1];
                    if (is_string($value[1])) {
                        $valArr = explode(",", trim($value[1]));
                    }
                    $model = $model->whereNotIn($column, $valArr);
                } elseif ($value[0] == 'null') {
                    $model = $model->whereNull($column);
                } elseif ($value[0] == 'not null') {
                    $model = $model->whereNotNull($column);
                } elseif ($value[0] !== '' || $value[1] !== '') {
                    $model = $model->whereBetween($column, $value);
                }
            } else {
                $model = $model->where($column, $value);
            }
        }
        if ($field) {
            $model = $model->orderBy($field, $order);
        }
        return $model;
    }

    public function afterQuery($items)
    {
        foreach ($items as &$item) {
            if (!empty($item['sheet'])) {
                $item['sheet']['goods_name'] = Goods::where('goods_id', $item['sheet']['join_sheet_goods_id'])->value('goods_name');
            }
        }

        return $items;
    }

    /**
     * @Desc 订单商品详情
     * @Author Gorden
     * @Date 2024/3/29 8:50
     *
     * @param Request $request
     * @return Response
     */
    public function sheet(Request $request)
    {
        $orderId = $request->get('order_id');
        $orderSheet = OrderSheet::with([
            'member' => function ($query) {
                $query->select('member_id', 'member_mobile');
            },
            'goods' => function ($query) {
                $query->select('goods_id', 'goods_name', 'goods_cover', 'goods_market_price', 'goods_sales_price', 'goods_classify');
            },
            'memberInfo',
            'cert'
        ])->where('join_sheet_order_id', $orderId)
            ->get()
            ->toArray();
        foreach ($orderSheet as &$item) {
            $item['goods']['goods_cover'] = getenv('STORAGE_DOMAIN') . $item['goods']['goods_cover'];
            if (!empty($item['goods']) && $item['goods']['goods_classify'] == 'PACKAGE') {
                $components = GoodsComponent::with('goods')
                    ->where('join_component_master_goods_id', $item['goods']['goods_id'])
                    ->select('join_component_master_goods_id', 'join_component_goods_id', 'goods_component_price',
                        'goods_component_price', 'goods_component_json')
                    ->get()
                    ->toArray();
                $goodsArr = [];
                foreach ($components as $component) {
                    $configJson = !empty($component['goods_component_json']) ? json_decode($component['goods_component_json'], true) : [];
                    if (!empty($component['goods'])) {
                        $supplierName = Supplier::where('supplier_id', $component['goods']['join_goods_supplier_id'])->value('supplier_name');
                        $goodsArr[] = [
                            'goods_name' => $component['goods']['goods_name'],
                            'goods_cover' => getenv('STORAGE_DOMAIN') . $component['goods']['goods_cover'],
                            'supplier_name' => $supplierName,
                            'nbr' => $configJson['nbr'] ?? 0,
                        ];
                    }
                }

                $item['goods']['components'] = $goodsArr;
            }
        }

        $order = Order::where('order_id', $orderId)->first();
        $express = OrderExpress::where('join_express_order_id', $orderId)->first();
        $data = [
            'order' => $order,
            'sheet' => $orderSheet,
            'express' => $express
        ];

        return json_success('', $data);
    }

//    protected function updateInput(Request $request): array
//    {
//        $primary_key = $this->model->getKeyName();
//        $id = $request->post($primary_key);
//        $data = $this->inputFilter($request->post());
//        $data['coupon_begindate'] = strtotime($data['coupon_begindate']);
//        $data['coupon_enddate'] = strtotime($data['coupon_enddate']);
//        $model = $this->model->find($id);
//        if (!$model) {
//            throw new BusinessException('记录不存在', 2);
//        }
//        unset($data[$primary_key]);
//        return [$id, $data];
//    }

    public function insert(Request $request): Response
    {
        $params = $request->post();
        try {
            Db::beginTransaction();
            // 余额支付
            if ($params['pay_category'] == 'CASH') {
                $account = MemberAccount::where('join_account_member_id', $params['join_order_member_id'])
                    ->where('member_account_classify', 'CASH')
                    ->where('member_account_status', 'ACTIVED')
                    ->first();
                if (!$account) {
                    return json_fail('账户异常');
                }
                $amount = $account->member_account_surplus + $account->member_account_added;
                if (!$account || $params['order_sheet_pay'] > $amount) {
                    return json_fail('账户余额不足');
                }
                if ($params['order_sheet_pay'] > $account->member_account_surplus) {
                    $cut = $account->member_account_added - ($params['order_sheet_pay'] - $account->member_account_surplus);
                    $account->member_account_surplus = 0;
                    $account->member_account_added = $cut;
                } else {
                    $account->member_account_surplus = $account->member_account_surplus - $params['order_sheet_pay'];
                }
                $account->member_account_expend = $account->member_account_expend + $params['order_sheet_pay'];
                $account->save();
            }

            $goods = Goods::where('goods_id', $params['join_sheet_goods_id'])
                ->select('goods_id', 'goods_name', 'goods_classify')
                ->first();
            if (!$goods) {
                return json_fail('产品数据异常');
            }
            $goods = $goods->toArray();

            $writeOffDate = [];
            // 服务已完成，生成核销数据
            if($params['order_status_system'] == 'DONE' && in_array($goods['goods_classify'],['SERVICE','PACKAGE'])){
                $writeOffDate = OrderService::generateWriteOffData($params);
            }

            $params['orderId'] = 'OD' . date('YmdHis') . random_string(6, 'up');
            $params['orderGroupId'] = 'OD' . date('YmdHis') . random_string(6, 'up');
            $params['benefitId'] = 'BF' . date('YmdHis') . random_string(6, 'up');

            // 配送方式
            if($params['delivery'] == 'PICKUP'){ // 自提
                $params['order_express_json'] = json_encode([
                    'express'=>'S',
                    'premises'=>$params['dept_premises_id']
                ]);
            }else if($params['delivery'] == 'LOGISTICS'){
                $params['order_express_json'] = json_encode([
                    'express'=>'Y',
                ]);
                $this->saveExpress($params);
            }

            // 写入主订单
            $this->insertMain($params);
            // 订单详情
            $this->insertSheet($params);
            // 支付记录
            $this->insertPayDetail($params);


            // 买的单个服务
            if ($goods['goods_classify'] == 'SERVICE') {
                // 预约表
                for ($i = 0; $i < intval($params['order_sheet_num']); $i++) {
                    $params['appointmentId'] = 'AP' . date('YmdHis') . random_string(6, 'up');
                    // 入预约记录
                    $this->insertAppointment($params,$writeOffDate);
                }
                $goods['skuId'] = $params['join_sheet_goods_sku_id'];
                $goods['category'] = 'SERVICE';
                // 权益表
                $this->insertMemberBenefit($params, $goods);
            } elseif ($goods['goods_classify'] == 'PACKAGE') {  // 一个套餐买多个
                $params['packageId'] = $params['join_sheet_goods_id'];
                $components = GoodsComponent::with([
                    'goods' => function ($query) {
                        $query->select('goods_id', 'goods_name', 'goods_classify');
                    }
                ])->where('join_component_master_goods_id', $params['join_sheet_goods_id'])
                    ->get()
                    ->toArray();
                $orderSheetNum = $params['order_sheet_num'];
                foreach ($components as $component) {
                    $params['order_sheet_num'] = $orderSheetNum;
                    $componentJson = json_decode($component['goods_component_json'], true);
                    $params['join_sheet_goods_sku_id'] = $componentJson['sku_id'];
                    $params['join_sheet_goods_id'] = $component['join_component_goods_id'];
                    $params['benefitId'] = 'BF' . date('YmdHis') . random_string(6, 'up');
                    $params['order_sheet_num'] = $params['order_sheet_num'] * $componentJson['nbr'];
                    for ($i = 0; $i < intval($params['order_sheet_num']); $i++) {
                        $params['appointmentId'] = 'AP' . date('YmdHis') . random_string(8, 'up');
                        // 入预约记录
                        $this->insertAppointment($params,$writeOffDate);
                    }
                    $goods['goods_id'] = $component['join_component_goods_id'];
                    $goods['goods_name'] = $component['goods']['goods_name'];
                    $goods['goods_classify'] = $component['goods']['goods_classify'];
                    $goods['skuId'] = $params['join_sheet_goods_sku_id'];
                    $goods['category'] = 'SERVICE';
                    // 权益表
                    $this->insertMemberBenefit($params, $goods);
                }
            }

            Db::commit();

            return json_success('创建订单成功');
        } catch (\Exception $e) {
            Db::rollBack();
            dump($e->getMessage());
            return json_fail('创建订单失败');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/6/7 10:30
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public function insertMain($params)
    {
        try {
            $data = [
                'order_id' => $params['orderId'],
                'order_groupby' => $params['orderGroupId'],
                'join_order_member_id' => $params['join_order_member_id'],
                'order_name' => date('Y-m-d H:i:s') . '-订单',
                'order_amount_total' => $params['order_sheet_amount'],
                'order_amount_pay' => $params['order_sheet_pay'],
                'order_category' => 'NORMAL',
                'order_status_system' => $params['order_status_system'],
                'order_status_payment' => $params['order_status_payment'],
                'order_status_storage' => $params['order_status_storage'],
                'order_remark' => $params['order_remark'] ?? '',
                'order_addtimes' => time()
            ];

            Order::insert($data);
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException('订单创建信息失败');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/6/7 10:25
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public function insertSheet($params)
    {
        try {
            $data = [
                'join_sheet_member_id' => $params['join_order_member_id'],
                'join_sheet_order_id' => $params['orderId'],
                'join_sheet_goods_id' => $params['join_sheet_goods_id'],
                'join_sheet_goods_sku_id' => $params['join_sheet_goods_sku_id'],
                'order_sheet_status' => 'DONE',
                'order_sheet_category' => 'NORMAL',
                'order_sheet_num' => $params['order_sheet_num'],
                'order_sheet_price' => $params['order_sheet_price'],
                'order_sheet_amount' => $params['order_sheet_amount'],
                'order_sheet_pay' => $params['order_sheet_pay'],
                'order_sheet_remark'=>$params['order_remark'] ?? '',
                'order_sheet_addtimes' => time()
            ];
            OrderSheet::insert($data);
        } catch (\Exception $e) {
            throw new BusinessException('订单创建失败');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/6/7 10:35
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public function insertPayDetail($params)
    {
        try {
            $data = [
                'join_pay_member_id' => $params['join_order_member_id'],
                'join_pay_order_id' => $params['orderGroupId'],
                'pay_status' => 'SUCCESS',
                'pay_category' => $params['pay_category'],
                'pay_amount' => $params['order_sheet_pay'],
                'pay_prepayid' => 0,
                'pay_paytimes' => date('Y-m-d H:i:s'),
                'pay_remark'=>$params['order_remark'] ?? '',
                'pay_addtimes' => time()
            ];

            PayDetail::insert($data);
        } catch (\Exception $e) {
            throw new BusinessException('创建支付记录失败');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/6/7 10:50
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public function insertAppointment($params,$writeOffDate)
    {
        try {
            $data = [
                'appointment_id' => $params['appointmentId'],
                'join_appointment_member_id' => $params['join_order_member_id'],
                'join_appointment_goods_id' => $params['join_sheet_goods_id'],
                'join_appointment_goods_sku_id' => $params['join_sheet_goods_sku_id'],
                'join_appointment_order_id' => $params['orderId'],
                'join_appointment_member_benefit_id' => $params['benefitId'],
                'appointment_classify' => 'SERVICE',
                'appointment_status' => $params['order_status_system'] == 'DONE' ? 'DONE' : 'INIT',
                'appointment_category' => 'NORMAL',
                'appointment_addtimes' => time(),
                'appointment_done_datetime'=>$params['order_status_system'] == 'DONE' ? date('Y-m-d H:i:s') : '',
                'appointment_remark' => $params['order_remark'] ?? '',
                'appointment_done_json' => $params['order_status_system'] == 'DONE' && $writeOffDate ? json_encode($writeOffDate) : '[]'
            ];
            Appointment::insert($data);
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException("创建权益记录失败");
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/6/7 11:05
     *
     * @param $params
     * @param $goods
     * @return void
     * @throws BusinessException
     */
    public function insertMemberBenefit($params, $goods)
    {
        try {
            $data = [
                'member_benefit_id' => $params['benefitId'],
                'join_benefit_member_id' => $params['join_order_member_id'],
                'join_benefit_package_id' => $params['packageId'] ?? '',
                'join_benefit_goods_id' => $goods['goods_id'],
                'join_benefit_goods_sku_id' => $goods['skuId'] ?? '',
                'join_benefit_order_id' => $params['orderId'],
                'member_benefit_status' => 'ACTIVED',
                'member_benefit_category' => $goods['category'],
                'member_benefit_name' => $goods['goods_name'],
                'member_benefit_limit_count' => $params['order_sheet_num'],
                'member_benefit_used_count' => $params['order_status_system'] == 'DONE' ? $params['order_sheet_num'] : 0,
                'member_benefit_remark' => $params['order_remark'] ?? '',
                'member_benefit_addtimes' => time()
            ];
            MemberBenefit::insert($data);
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException('创建会员权益失败');
        }
    }

    public function delivery(Request $request)
    {
        if (!Order::where('order_id', $request->post('order_id'))->where('order_status_system', 'SENDING')->exists()) {
            return json_fail('订单状态异常');
        }

        // 验证
        $validate = new OrderExpressValidate();
        if (!$validate->scene('delivery')->check($request->post())) {
            return json_fail($validate->getError());
        }
        try {
            Db::beginTransaction();
            // 该订单状态
            Order::where('order_id', $request->post('order_id'))->update(['order_status_system' => 'RECVING', 'order_status_storage' => 'EXPRESSING']);
            // 订单包含的产品
            $sheet = OrderSheet::where('join_sheet_order_id', $request->post('order_id'))
                ->select('join_sheet_goods_id')
                ->get();
            // 入配送记录表
            $params = $request->post();
            $model = OrderExpress::where('join_express_order_id',$params['order_id'])->first();
            $model->join_express_order_id = $params['order_id'];
            $model->order_express_goods = $sheet;
            $model->order_express_type = $params['order_express_type'];
            $model->order_express_code = $params['order_express_code'];
            $model->order_express_city = $params['order_express_city'];
            $model->order_express_address = $params['order_express_address'];
            $model->order_express_mobile = $params['order_express_mobile'];
            $model->order_express_telephone = $params['order_express_telephone'] ?? null;
            $model->order_express_person = $params['order_express_person'] ?? null;
            $model->order_express_remark = $params['order_express_remark'] ?? null;
            $model->order_express_addtimes = time();
            $model->save();

            // 15天后自动确认收货
            $redis = Redis::connection();
            $key = Order::AUTOMATIC_RECEIPT_PREFIX . date('Ymd', strtotime("+15 days"));
            $redis->sadd($key, $params['order_id']);

            Db::commit();

            return json_success('发货成功');
        } catch (\Exception $e) {
            Db::rollBack();

            return json_fail('发货失败');
        }
    }

    public function writeOff(Request $request){
        $memberId = $request->post('member_id', '');
        $benefitId = $request->post('benefit_id', '');
        $nbr = $request->post('nbr');
        $times = $request->post('times', '');
        $code = $request->post('sms_code', '');

        if (!$memberId || !$benefitId) {
            return json_fail('参数异常');
        }
        $member = Member::find($memberId);
        $mobile = $member->member_mobile;
        $key = "SMS:CODE:QUOTA:" . $mobile;
        $redisCode = Redis::get($key);
        if ($redisCode != $code) {
            return json_fail("验证码错误，请重新输入");
        }
        Redis::del($key);

        $benefit = MemberBenefit::where('member_benefit_id', $benefitId)
            ->where('join_benefit_member_id', $memberId)
            ->first();
        if ($benefit->member_benefit_limit_count - $benefit->member_benefit_used_count < $nbr) {
            return json_fail('可核销数量不足');
        }

        if (!$times) {
            $times = date('Y-m-d H:i:s');
        } else {
            $times = date('Y-m-d H:i:s', strtotime($times));
        }

        try {
            Db::beginTransaction();
            // 减额度
            $benefit->member_benefit_used_count = $benefit->member_benefit_used_count + $nbr;
            // 用完了
            if($benefit->member_benefit_limit_count <= $benefit->member_benefit_used_count){
                $benefit->member_benefit_status = 'DONE';
                if(!empty($benefit->join_benefit_order_id)){
                    Order::where('order_id',$benefit->join_benefit_order_id)->update(['order_status_system'=>'DONE']);
                }
            }

            $benefit->save();
            // 记录核销
            $params = [
                'dept_premises_id' => $request->post('dept_premises_id'),
                'order_remark'=>$request->post('remark'),
                'write_off_member_id' => $request->post('write_off_member_id'),
                'join_order_member_id' => $memberId
            ];
            $writeOffData = OrderService::generateWriteOffData($params);

            $appointments = Appointment::where('join_appointment_member_benefit_id', $benefitId)
                ->where('appointment_status', 'INIT')
                ->limit($nbr)
                ->get();
            foreach ($appointments as $appointment) {
                Appointment::where('appointment_id', $appointment->appointment_id)->update([
                    'appointment_status' => 'DONE',
                    'appointment_datetime' => $times,
                    'appointment_apply_datetime' => $times,
                    'appointment_doing_datetime' => $times,
                    'appointment_done_datetime' => $times,
                    'appointment_done_json' => json_encode($writeOffData)
                ]);
            }

            Db::commit();

            return json_success('核销成功');
        } catch (\Exception $e) {
            dump($e->getMessage());
            Db::rollBack();
            return json_fail('核销失败');
        }
    }

    public function myOrder(Request $request)
    {
        $memberId = $request->get('member_id', '');
        if (!$memberId) {
            return json_fail('查询错误');
        }

        $sheet = OrderSheet::with([
            'goods' => function ($query) {
                $query->select('goods_id', 'goods_name', 'goods_cover');
            },
            'order' => function ($query) {
                $query->select('order_id', 'order_status_system', 'order_status_payment', 'order_status_storage');
            }
        ])->where('join_sheet_member_id', $memberId)
            ->orderBy('order_sheet_addtimes', 'desc')
            ->get()
            ->toArray();
        foreach ($sheet as &$item) {
            if (!empty($item->goods->goods_cover)) {
                $item->goods->goods_cover = getenv('STORAGE_DOMAIN') . $item->goods->goods_cover;
            }
        }

        // 预约单
        $appointments = Db::table('appointment')
//            ->leftJoin('member', 'member.member_id', '=', 'appointment.join_appointment_member_id')
//            ->leftJoin('member_cert', 'member_cert.join_cert_member_id', '=', 'appointment.join_appointment_member_id')
//            ->leftJoin('member_info', 'member_info.join_info_member_id', '=', 'appointment.join_appointment_member_id')
            ->leftJoin('goods', 'goods.goods_id', '=', 'appointment.join_appointment_goods_id')
//            ->leftJoin('order', 'order.order_id', '=', 'appointment.join_appointment_order_id')
            ->leftJoin('member_benefit', 'member_benefit.member_benefit_id', '=', 'appointment.join_appointment_member_benefit_id')
//            ->leftJoin('goods_sku', 'goods_sku.goods_sku_id', '=', 'appointment.join_appointment_goods_sku_id')
            ->where('join_appointment_member_id', $memberId)
            ->select('appointment.*',
//            'member.member_mobile',
//            'member_cert.member_cert_name',
//            'member_info.member_info_nickname',
                'goods.goods_name', 'goods.goods_sales_price',
//            'order.order_name', 'goods_sku.goods_sku_specs_json', 'order.order_amount_total', 'order.order_amount_pay',
//            'order.order_category', 'order.order_status_system', 'order.order_status_payment', 'order.order_status_storage',
//            'order.order_addtimes',
                'member_benefit.member_benefit_name'
            )
            ->orderBy('appointment.appointment_addtimes', 'desc')
            ->get();
        foreach ($appointments as &$row) {
            if ($row->appointment_category == 'NOFEE') {
                $row->member_benefit_name = "后付费预约";
            }
            $row->appointment_addtimes = date('Y-m-d H:i:s', $row->appointment_addtimes);
        }

        return json_success('', ['order' => $sheet, 'appointment' => $appointments]);
    }

    public function saveExpress($params){
        try{
            $express = new OrderExpress();
            $express->join_express_order_id = $params['orderId'];
            $express->order_express_goods = json_encode(['sheet'=>'']);
            $express->order_express_city = $params['order_express_city'];
            $express->order_express_address = $params['order_express_address'];
            $express->order_express_mobile = $params['order_express_mobile'];
            $express->order_express_person = $params['order_express_person'];
            $express->order_express_addtimes = time();
            $express->save();
        }catch(\Exception $e){
            dump($e->getMessage());
            throw new BusinessException('物流信息保存失败');
        }
    }

    // TODO 确认收货（完成订单）
    public function confirm($id)
    {
        $order = Order::where('order_id', $id)->first();
        if (empty($order)) {
            return json_fail("暂无数据");
        }

        Db::beginTransaction();
        try {
            // 主订单
            Order::where('order_id', $id)->update(['order_status_system' => 'CONFIRM']);
        } catch (\Exception $e) {

        }
    }
}